//
//  MTLSegmentedControl.m
//  EnablerTest
//
//  Created by narita on 2013/01/18.
//  Copyright (c) 2013年 narita. All rights reserved.
//

#import "MTLSegmentedControl.h"

@interface MTLSegmentedControlTagedCellProxy : NSActionCell <MTLControlItemEnablerProtocol>
{
    NSInteger segmentNumber;
}

- (id) initWithTarget:(id)inTarget
               action:(SEL)inSelector
                 view:(NSView*)inView
        segmentNumber:(NSInteger)inNumber;
@end

@implementation MTLSegmentedControlTagedCellProxy

- (id) initWithTarget:(id)inTarget
               action:(SEL)inSelector
                 view:(NSView*)inView
        segmentNumber:(NSInteger)inNumber
{
    self = [super init];
    
    if(self)
    {
        [self setTarget:inTarget];
        [self setAction:inSelector];
        [self setControlView:inView];
        segmentNumber = inNumber;
    }
    
    return self;
}

- (NSSegmentedControl*) segmentedControl
{
    return (NSSegmentedControl*) [self controlView];
}

// --
- (void)setWidth:(CGFloat)width
{
    [[self segmentedControl] setWidth:width
                           forSegment:segmentNumber];
}

- (CGFloat)width
{
    return [[self segmentedControl] widthForSegment:segmentNumber];
}

// --
- (void)setImage:(NSImage *)image
{
    [[self segmentedControl] setImage:image
                           forSegment:segmentNumber];
}

- (NSImage *)image
{
    return [[self segmentedControl] imageForSegment:segmentNumber];
}

// --

- (void)setImageScaling:(NSImageScaling)scaling
{
    [[self segmentedControl] setImageScaling:scaling
                                  forSegment:segmentNumber];
}

- (NSImageScaling)imageScaling
{
    return [[self segmentedControl] imageScalingForSegment:segmentNumber];
}

// --

- (void)setLabel:(NSString *)label
{
    [[self segmentedControl] setLabel:label
                           forSegment:segmentNumber];
}

- (NSString *)label
{
    return [[self segmentedControl] labelForSegment:segmentNumber];
}

// --

- (void)setSelected:(BOOL)selected
{
    [[self segmentedControl] setSelected:selected
                              forSegment:segmentNumber];
}


- (BOOL)isSelected
{
    return [self selected];
}

- (BOOL) selected
{
    return [[self segmentedControl] isSelectedForSegment:segmentNumber];
}


// --

- (void) setState:(NSInteger)value
{
    [self setSelected:value];
}

- (NSInteger) state
{
    return [self selected];
}

- (NSInteger) tag
{
    //
    NSSegmentedCell* theCell = [[self segmentedControl] cell];
    
    return [theCell tagForSegment:segmentNumber];
}

// --

- (void) setEnabled:(BOOL)flag
{
    [[self segmentedControl] setEnabled:flag
                             forSegment:segmentNumber];
}

- (BOOL) isEnabled
{
    return [self enabled];
}

- (BOOL) enabled
{
    return [[self segmentedControl] isEnabledForSegment:segmentNumber];
}

@end


@implementation MTLSegmentedControl

#pragma mark -
#pragma mark class methods

#pragma mark -
#pragma mark init methods

#pragma -
#pragma mark dealloc

#pragma mark -
#pragma mark NSCopying, hash, isEqual:

#pragma mark -
#pragma mark NSCoding methods

#pragma mark -
#pragma mark awakeFromNib-like methods

#pragma mark -
#pragma mark action & canAction methods

#pragma mark -
#pragma mark delegate/datasource methods

- (void)viewDidMoveToWindow
{
    [super viewDidMoveToWindow];
    
    NSWindow* theWindow = [self window];
    
    if( theWindow != nil )
    {
        [[NSNotificationCenter defaultCenter] addObserver:self
                                                 selector:@selector(updateEnableState)
                                                     name:NSWindowDidUpdateNotification
                                                   object:theWindow];
    }
}

- (void)viewWillMoveToWindow:(NSWindow *)newWindow
{
    [super viewWillMoveToWindow:newWindow];
    
    NSWindow* theWindow = [self window];
    
    if( theWindow != nil )
    {
        [[NSNotificationCenter defaultCenter] removeObserver:self
                                                        name:NSWindowDidUpdateNotification
                                                      object:theWindow];
    }
    
}

#pragma mark -
#pragma mark accessor methods (in pairs)

- (NSInteger) selectedTag
{
    NSSegmentedCell* theCell = [self cell];
    
    return [theCell tagForSegment:[theCell selectedSegment]];
}

#pragma mark -
#pragma mark Utility methods

- (void) updateEnableState
{
    NSInteger theCount = [self segmentCount];
    id 		  theValidator;

    theValidator = [NSApp targetForAction:[self action]
                                       to:[self target]
                                     from:self];
    
    for(NSInteger i = 0; i < theCount; i++)
    {
        MTLSegmentedControlTagedCellProxy* theCell;
        
        theCell = [[[MTLSegmentedControlTagedCellProxy alloc] initWithTarget:[self target]
                                                                      action:[self action]
                                                                        view:self
                                                               segmentNumber:i] autorelease];
        
        if ((theValidator == nil) || ![theValidator respondsToSelector:[self action]])
        {
            [theCell setEnabled:NO];
        }
		else if ([theValidator respondsToSelector:@selector(validateUserInterfaceItem:)])
		{
			// actionを実行できるレスポンダーが"validateUserInterfaceItem:"を実装しているならば従う
			[theCell setEnabled:
             [(id<NSUserInterfaceValidations>)theValidator validateUserInterfaceItem:
              (id<NSValidatedUserInterfaceItem>)theCell]];
		}
		else
		{
			// レスポンダーがあるけど、"validateXXXX"系のメソッドが無い場合は、ボタンは押せる。
			[theCell setEnabled:YES];
		}
        
    }
    
    // Controlのアップデート
	// actionが設定されているかをチェック。(target == nil)の場合はfirstResponderだから有効なのでtargetのnilチェックはしない。
    if( [self action] != nil && [self window] != nil && [self isHiddenOrHasHiddenAncestor] == NO)
	{
		id 			theValidator;
		
		// レスポンダーチェーンをたどって実際にactionを処理するオブジェクトを探す。そのオブジェクトがメニューやボタンのenable/disableの値を決定出来るはず
		theValidator = [NSApp targetForAction:[self action]
										   to:[self target]
										 from:self];
		
		
		if([self infoForBinding:@"enabled"])
		{
			// enabledにCocoaBindingが設定されている場合は
			// CocoaBindingの設定を優先させるために何もしない。
		}
		else if ((theValidator == nil) || ![theValidator respondsToSelector:[self action]])
		{
			// actionを実行できるレスポンダーが無い場合は、ボタンは押せない
			[self setEnabled:NO];
		}
		else if ([theValidator respondsToSelector:@selector(validateToolbarItem:)])
		{
			// actionを実行できるレスポンダーが"validateToolbarItem:"を実装しているならば従う
			[self setEnabled:
             [theValidator validateToolbarItem:
              (NSToolbarItem*)self]];
		}
		else if ([theValidator respondsToSelector:@selector(validateUserInterfaceItem:)])
		{
			// actionを実行できるレスポンダーが"validateUserInterfaceItem:"を実装しているならば従う
			[self setEnabled:
             [(id<NSUserInterfaceValidations>)theValidator validateUserInterfaceItem:
              (id<NSValidatedUserInterfaceItem>)self]];
		}
		else
		{
			// レスポンダーがあるけど、"validateXXXX"系のメソッドが無い場合は、ボタンは押せる。
			[self setEnabled:YES];
		}
	}
}

@end
